@@ -6,6 +6,7 @@ import cryolo.__init__ as ini
 
 
 def config_boxmanager_parser(parser_config):
+
     config_required_group = parser_config.add_argument_group(
         "General arguments", "Options to start the box manager.",
     )
@@ -22,6 +23,10 @@ def config_boxmanager_parser(parser_config):
         "--wildcard", help="Wildcard for selecting specific images (e.g *_new_*.mrc)"
     )
 
+    config_required_group.add_argument("-t", "--is_tomo_dir", action="store_true",
+                           help="Flag for specifying that the directory contains tomograms.")
+
+
 
 def config_configuration_parser(parser_config):
     config_required_group = parser_config.add_argument_group(
@@ -291,7 +296,7 @@ def create_parser():
     # Config generator
     parser_config = subparsers.add_parser(
         "config",
-        help="train help",
+        help="Command to create the configuration file",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
     config_configuration_parser(parser_config)
@@ -299,7 +304,7 @@ def create_parser():
     # Training parser
     parser_train = subparsers.add_parser(
         "train",
-        help="train help",
+        help="Command to train crYOLO",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
 
@@ -310,7 +315,7 @@ def create_parser():
     # Picking parser
     parser_pick = subparsers.add_parser(
         "predict",
-        help="predict help",
+        help="Command to pick particles using a trained model.",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
 
@@ -322,7 +327,7 @@ def create_parser():
     # Evaluation parser
     parser_eval = subparsers.add_parser(
         "evaluation",
-        help="evalutation help",
+        help="Command to evaluate the picking.",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
     from cryolo.eval import create_parser as create_eval_parser
@@ -334,7 +339,7 @@ def create_parser():
     # crYOLO boxmanager
     parser_config = subparsers.add_parser(
         "boxmanager",
-        help="boxmanager help",
+        help="Display boxes",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
     config_boxmanager_parser(parser_config)
@@ -428,7 +433,7 @@ def main():
     elif "boxmanager" in sys.argv[1]:
         from cryoloBM import boxmanager
 
-        boxmanager.start_boxmanager(args.image_dir, args.box_dir, args.wildcard)
+        boxmanager.start_boxmanager(args.image_dir, args.box_dir, args.wildcard, is_tomo=args.is_tomo_dir)
 
 
 def _main_():
