@@ -37,7 +37,6 @@ import cryolo.lowpass as lowpass
 import cryolo.preprocessing as preprocess
 from . import utils
 from gooey import Gooey, GooeyParser
-from distutils import util as distutils
 
 os.environ["CUDA_DEVICE_ORDER"] = "PCI_BUS_ID"
 try:
@@ -157,6 +156,15 @@ def create_parser(parser):
         help="If true, it will delete the filtered images after training is done."
     )
 
+    optional_group.add_argument(
+        "--ignore_directions",
+        action="store_true",
+        default=False,
+        help="When using filament training data, crYOLO will automatically detect that and train the"
+             "model in wtih directional estimation included. If you use this option, this directional"
+             "learning is skipped.",
+    )
+
     deexsp_group = parser.add_argument_group(
         "Deprecated/Experimental/Special ",
         "Contains either deprecated / experimental or very special options.",
@@ -203,6 +211,7 @@ def get_model(
     num_fine_tune_layers,
     num_gpus,
     pretrained_weights,
+    is_filament_training
 ):
     """
     Create the model for training/prediction
@@ -228,6 +237,7 @@ def get_model(
     # import tensorflow as tf
     # graph = tf.Graph()
     # with graph.as_default():
+
     if num_gpus > 1:
         import tensorflow as tf
 
@@ -242,8 +252,10 @@ def get_model(
                 backend_weights=backend_weights,
                 experimental_loss=experimental_loss,
                 pretrained_weights=pretrained_weights,
+                pretrained_weights_init=True,
                 fine_tune=do_fine_tune,
                 num_fine_tune_layers=num_fine_tune_layers,
+                filament_model=is_filament_training
             )
     else:
         yolo = YOLO(
@@ -256,8 +268,10 @@ def get_model(
             backend_weights=backend_weights,
             experimental_loss=experimental_loss,
             pretrained_weights=pretrained_weights,
+            pretrained_weights_init=True,
             fine_tune=do_fine_tune,
             num_fine_tune_layers=num_fine_tune_layers,
+            filament_model=is_filament_training
         )
 
     # print a summary of the whole model
@@ -313,6 +327,10 @@ def main(args=None):
     Main function. Read the arguments, start the program :-)
     :return: None
     """
+    #import os
+    import logging
+    #logging.getLogger('tensorflow').disabled = True
+    logging.getLogger('tensorflow').setLevel(logging.ERROR)
     try:
         multiprocessing.set_start_method("spawn")
     except RuntimeError:
@@ -348,20 +366,20 @@ def main(args=None):
         import tensorflow as tf
         from keras.backend.tensorflow_backend import set_session
 
-        config = tf.ConfigProto()
+        config = tf.compat.v1.ConfigProto()
 
         config.gpu_options.allow_growth = True
         config.gpu_options.per_process_gpu_memory_fraction = args.gpu_fraction
 
-        set_session(tf.Session(config=config))
+        set_session(tf.compat.v1.Session(config=config))
     else:
         import tensorflow as tf
         from keras.backend.tensorflow_backend import set_session
 
-        config = tf.ConfigProto()
+        config = tf.compat.v1.ConfigProto()
         config.gpu_options.allow_growth = True
 
-        set_session(tf.Session(config=config))
+        set_session(tf.compat.v1.Session(config=config))
 
     early_stop = int(args.early)
     do_multiprocessing = False
@@ -398,7 +416,7 @@ def main(args=None):
         log_path = config["other"]["log_path"]
     else:
         log_path = "logs_cryolo/"
-        print("")
+
     if not os.path.exists(log_path):
         try:
             os.makedirs(log_path)
@@ -451,6 +469,8 @@ def main(args=None):
     if early_stop < warmup_epochs:
         early_stop = warmup_epochs
 
+    # Check if directional estimation should ignore
+    ignore_directions = args.ignore_directions
     ###############################
     #   Parse the annotations
     ###############################
@@ -459,9 +479,9 @@ def main(args=None):
     train_annot_pairs = preprocess.find_image_annotation_pairs_by_dir(os.path.realpath(config["train"]["train_annot_folder"]),
                                                                       os.path.realpath(config["train"]["train_image_folder"]))
 
+
     # Identifiy all image sizes:
     sizes = preprocess.get_image_size_distr([pair[0] for pair in train_annot_pairs])
-
     cryolo_mode = utils.get_cryolo_mode(size_distr=sizes, config_input_size=config["model"]["input_size"])
 
     print("###############################################")
@@ -469,36 +489,40 @@ def main(args=None):
     for size in sizes:
         print(size[0], "x", size[1], "( N:",size[2],")")
     print("")
-    print("crYOLO will train in mode:", cryolo_mode)
+    print("crYOLO will train in mode:", cryolo_mode.name)
 
-
-
-
-    if cryolo_mode == "NON-SQUARE":
+    config["model"]["input_size"] = config_tools.get_adjusted_input_size(config)
+    if cryolo_mode == utils.CryoloMode.NON_SQUARE:
         width = sizes[0][0]
         height = sizes[0][1]
         ar = height/width
         if ar < 1:
-            input_size = [config["model"]["input_size"],int(config["model"]["input_size"] / ar)]
+            input_size = [config["model"]["input_size"],config["model"]["input_size"] / ar]
         else:
-            input_size = [int(config["model"]["input_size"] * ar),config["model"]["input_size"]]
-
+            input_size = [config["model"]["input_size"] * ar,config["model"]["input_size"]]
         adjusted_size = config_tools.adjust_size(input_size)
         print("Your training input size",config["model"]["input_size"],"was adjusted to",adjusted_size,"(height, width)")
         config["model"]["input_size"]= adjusted_size
-    else:
-        config["model"]["input_size"] = config_tools.get_adjusted_input_size(config)
 
     print("###############################################")
 
     grid_w, grid_h = config_tools.get_gridcell_dimensions(config)
 
     # parse annotations of the training set
-    train_imgs, train_labels = preprocess.parse_annotation(
+    parse_dict = preprocess.parse_annotation(
         train_annot_pairs,
         grid_dims=(grid_w, grid_h, num_patches),
         anchor_size=int(config["model"]["anchors"][0]),
     )
+    train_imgs = parse_dict["images"]
+
+    max_part = np.max([len(img["object"]) for img in train_imgs])
+    is_filament_data = parse_dict["is_filament_data"]
+
+    if max_part > config["model"]["max_box_per_image"]:
+        print("The value for max_box_per_image", config["model"]["max_box_per_image"], "is to small for the given training data.")
+        config["model"]["max_box_per_image"] = int(max_part*1.1)
+        print("Set max_box_per_image to", config["model"]["max_box_per_image"])
 
     config["model"]["labels"] = ["particle"]
 
@@ -514,12 +538,13 @@ def main(args=None):
             os.path.realpath(config["valid"]["valid_annot_folder"]),
             os.path.realpath(config["valid"]["valid_image_folder"]))
 
-        valid_imgs, valid_labels = preprocess.parse_annotation(
+        parse_dict = preprocess.parse_annotation(
             valid_annot_pairs,
             grid_dims=(grid_w, grid_h, num_patches),
             anchor_size=int(config["model"]["anchors"][0]),
         )
-
+        valid_imgs = parse_dict["images"]
+        valid_labels = parse_dict["labels"]
         if (
             len(valid_imgs) == 0
             or len(valid_labels) == 0
@@ -534,21 +559,36 @@ def main(args=None):
                     "No validation labels were found. Invalid validation configuration. Check your config file."
                 )
     else:
-        np.random.seed(seed)
-        train_valid_split = int(0.8 * len(train_imgs))
-        np.random.shuffle(train_imgs)
-        valid_imgs = train_imgs[train_valid_split:]
-        train_imgs = train_imgs[:train_valid_split]
-        print("Validation set:", len(valid_imgs),"images")
 
+        np.random.seed(seed)
+        images_picked = [img for img in train_imgs if len(img["object"])>0]
+        images_empty = [img for img in train_imgs if len(img["object"]) == 0]
+
+        train_valid_split_picked = int(0.8 * len(images_picked))
+        np.random.shuffle(images_picked)
+        valid_imgs = images_picked[train_valid_split_picked:]
+        train_imgs = images_picked[:train_valid_split_picked]
+
+
+        if len(images_empty)>0:
+            train_valid_split_empty = int(0.8 * len(images_empty))
+            np.random.shuffle(images_empty)
+            valid_imgs.extend(images_empty[train_valid_split_empty:])
+            train_imgs.extend(images_empty[:train_valid_split_empty])
+
+    if len(train_imgs) == 0:
+        import sys
+        print("Training data size is too small. Please pick at least two images / slices.")
+        sys.exit(0)
+    print("Validation set:", len(valid_imgs), "images")
     #####################################
     # Write runfile
     #####################################
 
     valid_imgs_paths = [item["filename"] for item in valid_imgs]
-    valid_annot_paths = [item["boxpath"] for item in valid_imgs]
+    valid_annot_paths = [{"path":item["boxpath"]} if "z" not in item else {"path:":item["boxpath"], "slice":item["z"]} for item in valid_imgs]
     train_imgs_paths = [item["filename"] for item in train_imgs]
-    train_annot_paths = [item["boxpath"] for item in train_imgs]
+    train_annot_paths = [{"path":item["boxpath"]} if "z" not in item else {"path": item["boxpath"], "slice": item["z"]} for item in train_imgs]
 
     runjson = {}
     runjson["run"] = {}
@@ -582,8 +622,8 @@ def main(args=None):
             model_path, overlap, nn_batch_size, filter_img_path = filter_options
             print("Filter data using noise2noise model: ", model_path)
 
-            # Correct absolute filenames
-            train_imgs_paths = [img["filename"] for img in train_imgs]
+            # Get set of unique filenames
+            train_imgs_paths = list(set([img["filename"] for img in train_imgs]))
 
             filtered_paths = utils.filter_images_noise2noise_dir(
                 img_paths=train_imgs_paths,
@@ -595,10 +635,10 @@ def main(args=None):
             )
 
             # Update paths of newly filtered images
-            for index, path in enumerate(filtered_paths):
-                train_imgs[index]["filename"] = path
+            for img in train_imgs:
+                img["filename"] = filtered_paths[train_imgs_paths.index(img["filename"])]
 
-            valid_imgs_paths = [img["filename"] for img in valid_imgs]
+            valid_imgs_paths = list(set([img["filename"] for img in valid_imgs]))
             filtered_paths = utils.filter_images_noise2noise_dir(
                 img_paths=valid_imgs_paths,
                 output_dir_filtered_imgs=filter_img_path,
@@ -607,9 +647,11 @@ def main(args=None):
                 batch_size=nn_batch_size,
                 resize_to=resize_to,
             )
+
             # Update paths of newly filtered images
-            for index, path in enumerate(filtered_paths):
-                valid_imgs[index]["filename"] = path
+            for img in valid_imgs:
+                img["filename"] = filtered_paths[valid_imgs_paths.index(img["filename"])]
+
 
         else:
             cutoff, filter_img_path = filter_options
@@ -628,24 +670,28 @@ def main(args=None):
                 num_cpu=num_cpus,
                 resize_to_shape=resize_to,
             )
+
             end = time.time()
             print("Time needed for filtering:", end - start)
 
     # Read first image and check the image depth.
-    img_first = imagereader.image_read(train_imgs[0]["filename"])
+    img_first = imagereader.image_read(train_imgs[0]["filename"], use_mmap=True)
+
     isgrey = False
     if len(img_first.shape) == 2:
         isgrey = True
-    elif np.all(img_first[:, :, 0] == img_first[:, :, 1]) and np.all(
-        img_first[:, :, 0] == img_first[:, :, 2]
+    elif img_first.shape[0]==3 and np.all(img_first[:, :, 0] == img_first[:, :, 1]) and np.all(img_first[:, :, 0] == img_first[:, :, 2]
     ):
         isgrey = True
+    elif len(img_first.shape)==3 and img_first.shape[0]>3:
+        # Tomo data
+        img_first = img_first[0,:,:]
+        isgrey = True
 
     if isgrey:
         depth = 1
     else:
         depth = 3
-
     # As only on box size is expected, the anchor box size automatically
     anchors = utils.get_anchors(config, image_sizes=sizes)#
 
@@ -663,12 +709,21 @@ def main(args=None):
         normalization_string = config["model"]["norm"]
 
     ###############################
-    # Start the training process
+    # Start the training processn setup.
     ###############################
     pretrained_weights = None
     if os.path.exists(config["train"]["pretrained_weights"]):
         pretrained_weights = config["train"]["pretrained_weights"]
 
+    if ignore_directions and is_filament_data:
+        print("Ignoring directional filament information.")
+    is_filament_model = is_filament_data and ignore_directions==False
+
+    if is_filament_model:
+        print("#############################")
+        print("Filaments are used fore training. crYOLO will learn to estimate their directions "
+              "directly. If you don't want this, use the --ignore_directions option.")
+        print("#############################")
     import tensorflow as tf
 
     start = time.time()
@@ -682,6 +737,7 @@ def main(args=None):
         num_fine_tune_layers,
         num_gpus,
         pretrained_weights,
+        is_filament_model
     )
 
     # USE MULTIGPU
@@ -719,7 +775,9 @@ def main(args=None):
             num_cpus=num_cpus,
             do_augmentation=do_augmentation,
             do_multiprocessing=do_multiprocessing,
-            normalization=normalization_string
+            normalization=normalization_string,
+            cryolo_mode=cryolo_mode,
+            filament_model=is_filament_model,
         )
     except tf.errors.ResourceExhaustedError:
         print("############################")
